local VERSION = tonumber(("$Revision: 44396 $"):match("%d+"))

local Parrot = Parrot
local Parrot_AnimationStyles = Parrot:NewModule("AnimationStyles")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-20 22:43:47 -0400 (Fri, 20 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_AnimationStyles")

local animationStyles = {}
local animationStylesValidate = {}

--[[----------------------------------------------------------------------------------
Arguments:
	table - a data table holding the details of an animation style.
Notes:
	The data table is of the following style:
	<pre>{
		name = "Name of the style in English",
		localName = "Name of the style in the current locale",
		func = function(
			fontstring, -- the FontString to work with
			scrollArea, -- the scrollArea data table
			percent, -- [0, 1] the current percentage done.
			downward, -- boolean - whether to scroll downward instead of upward.
			num, -- the number of the FontString out of the others using the same area.
			max, -- the number of FontStrings using the same area
			uid, -- the unique ID of the FontString, for the given area.
		)
			fontstring:SetPoint(...) -- the FontString must be positioned here.
		end,
		init = function( -- this is optional
			fontstring, -- the FontString to work with
			scrollArea, -- the scrollArea data table
			downward, -- boolean - whether to scroll downward instead of upward.
			uid, -- the unique ID of the FontString, for the given area.
		)
			-- this is called before the first func() is called.
		end,
		cleanup = function( -- this is optional
			fontstring, -- the FontString to work with
			scrollArea, -- the scrollArea data table
			downward, -- boolean - whether to scroll downward instead of upward.
			uid, -- the unique ID of the FontString, for the given area.
		)
			-- this is called after the last func() is called.
		end,
		overlap = true, -- or false. This dictates whether FontStrings should be moved along prematurely if they are overlapping.
	}</pre>
Example:
	Parrot:RegisterAnimationStyle {
		-- simple vertical
		name = "Straight", 
		localName = L["Straight"],
		func = function(fontstring, scrollArea, percent, downward)
			local fsHeight = fontstring:GetHeight()
			local height = scrollArea.size - fsHeight
			local middle = scrollArea.yOffset
			local bottom = middle - scrollArea.size/2
			local center = scrollArea.xOffset
			if downward then
				percent = 1 - percent
			end
			local y = bottom + percent * height
			local x = center
			local align = scrollArea.align
			local point
			if align == "LEFT" then
				point = "BOTTOMLEFT"
			elseif align == "RIGHT" then
				point = "BOTTOMRIGHT"
			else -- center
				point = "BOTTOM"
			end
			fontstring:SetPoint(point, UIParent, "CENTER", x, y)
		end,
		overlap = true,
	}
------------------------------------------------------------------------------------]]
function Parrot_AnimationStyles:RegisterAnimationStyle(data)
	self = Parrot_AnimationStyles -- so people can Parrot:RegisterAnimationStyle
	if type(data) ~= "table" then
		error(("Bad argument #2 to `RegisterAnimationStyle'. Expected %q, got %q."):format("table", type(data)), 2)
	end
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterAnimationStyle'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterAnimationStyle'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	if type(data.func) ~= "function" then
		error(("Bad argument #2 to `RegisterAnimationStyle'. func must be a %q, got %q."):format("function", type(data.func)), 2)
	end
	if data.cleanup and type(data.cleanup) ~= "function" then
		error(("Bad argument #2 to `RegisterAnimationStyle'. cleanup must be a %q or %q, got %q."):format("function", "nil", type(data.cleanup)), 2)
	end
	if data.init and type(data.init) ~= "function" then
		error(("Bad argument #2 to `RegisterAnimationStyle'. init must be a %q or %q, got %q."):format("function", "nil", type(data.init)), 2)
	end
	if data.overlap and data.overlap ~= true then
		error(("Bad argument #2 to `RegisterAnimationStyle'. overlap must be a %q, got %q."):format("boolean", type(overlap)), 2)
	end
	if animationStyles[name] then
		error(("Animation style %q already registered"):format(name), 2)
	end
	animationStyles[name] = data
	animationStylesValidate[name] = localName
end
Parrot.RegisterAnimationStyle = Parrot_AnimationStyles.RegisterAnimationStyle

--[[----------------------------------------------------------------------------------
Arguments:
	string - the name of the animation style, in English.
Returns:
	boolean - whether the animation style has been registered or not.
Example:
	local has = Parrot:HasAnimationStyle("My funky style")
------------------------------------------------------------------------------------]]
function Parrot_AnimationStyles:HasAnimationStyle(name)
	self = Parrot_AnimationStyles -- so people can Parrot:RegisterAnimationStyle
	AceLibrary.argCheck(self, name, 2, "string")
	return not not animationStyles[name]
end
Parrot.HasAnimationStyle = Parrot_AnimationStyles.HasAnimationStyle

-- #NODOC
function Parrot_AnimationStyles:GetAnimationStyle(name)
	return animationStyles[name] or animationStyles.Straight
end

-- #NODOC
function Parrot_AnimationStyles:GetAnimationStylesValidate(name)
	return animationStylesValidate
end

------------------------------------------------------------------------------
-- Animation style registry --------------------------------------------------
------------------------------------------------------------------------------

Parrot_AnimationStyles:RegisterAnimationStyle {
	-- simple vertical
	name = "Straight", 
	localName = L["Straight"],
	func = function(fontstring, scrollArea, percent, downward)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local bottom = middle - scrollArea.size/2
		local center = scrollArea.xOffset
		if downward then
			percent = 1 - percent
		end
		local y = bottom + percent * height
		local x = center
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	overlap = true,
}
Parrot_AnimationStyles:RegisterAnimationStyle {
	-- makes a parabola in the form of x = y^2 - 1
	name = "Left parabola",
	localName = L["Left parabola"],
	func = function(fontstring, scrollArea, percent, downward)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local bottom = middle - scrollArea.size/2
		local center = scrollArea.xOffset
		if downward then
			percent = 1 - percent
		end
		local y = bottom + percent * height
		local x = center
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		x = x - (1 - 4*(percent - 0.5)^2) * height * 1/3
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	overlap = true,
}
Parrot_AnimationStyles:RegisterAnimationStyle {
	-- makes a parabola in the form of x = -y^2 + 1
	name = "Right parabola",
	localName = L["Right parabola"],
	func = function(fontstring, scrollArea, percent, downward)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local bottom = middle - scrollArea.size/2
		local center = scrollArea.xOffset
		if downward then
			percent = 1 - percent
		end
		local y = bottom + percent * height
		local x = center
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		x = x + (1 - 4*(percent - 0.5)^2) * height * 1/3
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	overlap = true,
}

Parrot_AnimationStyles:RegisterAnimationStyle {
	-- makes a semicircle in the form of x = -(-y^2 + 1)^0.5
	name = "Left semicircle",
	localName = L["Left semicircle"],
	func = function(fontstring, scrollArea, percent, downward)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local center = scrollArea.xOffset
		if downward then
			percent = 1 - percent
		end
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		local y = middle - height/2 * sin(90 + 180 * (percent))
		local x = center + height/2 * cos(90 + 180 * (percent))
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	overlap = true
}
Parrot_AnimationStyles:RegisterAnimationStyle {
	-- makes a semicircle in the form of x = (-y^2 + 1)^0.5
	name = "Right semicircle",
	localName = L["Right semicircle"],
	func = function(fontstring, scrollArea, percent, downward)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local center = scrollArea.xOffset
		if downward then
			percent = 1 - percent
		end
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		local y = middle - height/2 * sin(90 - 180 * (percent))
		local x = center + height/2 * cos(90 - 180 * (percent))
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	overlap = true,
}

Parrot_AnimationStyles:RegisterAnimationStyle {
	-- places in the center and shakes around
	name = "Pow",
	localName = L["Pow"],
	func = function(fontstring, scrollArea, percent, downward, num, max)
		local fsHeight = fontstring:GetHeight()
		local middle = scrollArea.yOffset
		local center = scrollArea.xOffset
		local topMiddle
		local topDiff = (max - 1) * 0.5 * fsHeight
		local currDiff = -(num - 1) * fsHeight
		local y
		if not downward then
			y = middle + topDiff + currDiff
		else
			y = middle - topDiff - currDiff
		end
	
		local x = center
		local point = scrollArea.align
		-- now for squiggle
		local time = GetTime()
		if not fontstring.nextSquiggle or fontstring.nextSquiggle < time then
			fontstring.nextSquiggle = time + 0.05
			fontstring.squiggleX = math.random(-1, 1)
			fontstring.squiggleY = math.random(-1, 1)
		end	
		x = x + fontstring.squiggleX
		y = y + fontstring.squiggleY
	
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	cleanup = function(fontstring, scrollArea)
		fontstring.nextSquiggle = nil
		fontstring.squiggleX = nil
		fontstring.squiggleY = nil
	end
}

Parrot_AnimationStyles:RegisterAnimationStyle {
	-- places in the center
	name = "Static",
	localName = L["Static"],
	func = function(fontstring, scrollArea, percent, downward, num, max)
		local fsHeight = fontstring:GetHeight()
		local middle = scrollArea.yOffset
		local center = scrollArea.xOffset
		local topMiddle
		local topDiff = (max - 1) * 0.5 * fsHeight
		local currDiff = -(num - 1) * fsHeight
		local y
		if not downward then
			y = middle + topDiff + currDiff
		else
			y = middle - topDiff - currDiff
		end
	
		local x = center
		local point = scrollArea.align
	
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end
}

local rr = {
	-- makes a parabola in the form of y = (-x - 1)(-x + 0.5) + 1
	name = "Right rainbow",
	localName = L["Right rainbow"],
	init = function(fontstring, scrollArea, downward, uid)
		fontstring.leftRoot = 1 + (math.random()-0.5) / 5
		fontstring.rightRoot = -0.5 + (math.random()-0.5) / 5
	end,
	func = function(fontstring, scrollArea, percent, downward, num, max, uid)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local center = scrollArea.xOffset
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		x = center + height/2 * percent
		y = height * -16/9 * (percent - fontstring.leftRoot) * (percent - fontstring.rightRoot)
		if not downward then
			y = -y
		end
		y = middle + y
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	cleanup = function(fontstring, scrollArea, downward, uid)
		fontstring.leftRoot = nil
		fontstring.rightRoot = nil
	end,
	overlap = false,
}
Parrot_AnimationStyles:RegisterAnimationStyle(rr)

local lr = {
	-- makes a parabola in the form of y = (-x - 1)(-x + 0.5) + 1
	name = "Left rainbow",
	localName = L["Left rainbow"],
	init = function(fontstring, scrollArea, downward, uid)
		fontstring.leftRoot = 1 + (math.random()-0.5) / 5
		fontstring.rightRoot = -0.5 + (math.random()-0.5) / 5
	end,
	func = function(fontstring, scrollArea, percent, downward, num, max, uid)
		local fsHeight = fontstring:GetHeight()
		local height = scrollArea.size - fsHeight
		local middle = scrollArea.yOffset
		local center = scrollArea.xOffset
		local align = scrollArea.align
		local point
		if align == "LEFT" then
			point = "BOTTOMLEFT"
		elseif align == "RIGHT" then
			point = "BOTTOMRIGHT"
		else -- center
			point = "BOTTOM"
		end
		x = center - height/2 * percent
		y = height * -16/9 * (percent - fontstring.leftRoot) * (percent - fontstring.rightRoot)
		if not downward then
			y = -y
		end
		y = middle + y
		fontstring:SetPoint(point, UIParent, "CENTER", x, y)
	end,
	cleanup = function(fontstring, scrollArea, downward, uid)
		fontstring.leftRoot = nil
		fontstring.rightRoot = nil
	end,
	overlap = false,
}
Parrot_AnimationStyles:RegisterAnimationStyle(lr)

Parrot_AnimationStyles:RegisterAnimationStyle {
	name = "Alternating rainbow",
	localName = L["Alternating rainbow"],
	init = function(fontstring, scrollArea, downward, uid)
		if uid%2 == 0 then
			return lr.init(fontstring, scrollArea, downward, uid)
		else
			return rr.init(fontstring, scrollArea, downward, uid)
		end
	end,
	func = function(fontstring, scrollArea, percent, downward, num, max, uid)
		if uid%2 == 0 then
			return lr.func(fontstring, scrollArea, percent, downward, num, max, uid)
		else
			return rr.func(fontstring, scrollArea, percent, downward, num, max, uid)
		end
	end,
	cleanup = function(fontstring, scrollArea, downward, uid)
		if uid%2 == 0 then
			return lr.cleanup(fontstring, scrollArea, downward, uid)
		else
			return rr.cleanup(fontstring, scrollArea, downward, uid)
		end
	end,
	overlap = false
}
